package com.usky.iot.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.iot.domain.DmpProductInfo;
import com.usky.iot.mapper.DmpProductInfoMapper;
import com.usky.iot.service.DmpDeviceInfoService;
import com.usky.iot.service.DmpDeviceStatusService;
import com.usky.iot.service.DmpProductInfoService;
import com.usky.iot.service.vo.DmpProductInfoRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 产品信息表 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-10-08
 */
@Service
public class DmpProductInfoServiceImpl extends AbstractCrudService<DmpProductInfoMapper, DmpProductInfo> implements DmpProductInfoService {


    @Autowired
    private DmpDeviceInfoService dmpDeviceInfoService;

    @Autowired
    private DmpDeviceStatusService dmpDeviceStatusService;
    @Override
    public boolean add(DmpProductInfo dmpProductInfo) {
        if (checkNameUnique(dmpProductInfo)){
            throw new BusinessException("新增产品信息'" + dmpProductInfo.getProductName() + "'失败，产品信息已存在");
        }
        return this.save(dmpProductInfo);
    }

    @Override
    public void update(DmpProductInfo dmpProductInfo) {
        dmpProductInfo.setUpdatedTime(new Date());
        if (checkNameUnique(dmpProductInfo)){
            throw new BusinessException("修改产品信息'" + dmpProductInfo.getProductName() + "'失败，产品信息已存在");
        }
        this.updateById(dmpProductInfo);
    }

    @Override
    public boolean checkNameUnique(DmpProductInfo dmpProductInfo) {
        Integer id = null == dmpProductInfo.getId() ? -1 : dmpProductInfo.getId();
        LambdaQueryWrapper<DmpProductInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DmpProductInfo::getProductName,dmpProductInfo.getProductName())
                .eq(DmpProductInfo::getDeleteFlag,0);
        DmpProductInfo one = this.getOne(queryWrapper);
        return null != one && !Objects.equals(one.getId(), id);
    }


    @Override
    public CommonPage<DmpProductInfo> page(DmpProductInfoRequest piRequest) {
        IPage<DmpProductInfo> page = new Page<>(piRequest.getCurrent(), piRequest.getSize());
        LambdaQueryWrapper<DmpProductInfo> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery
                .like(StringUtils.isNotBlank(piRequest.getProductName()),DmpProductInfo::getProductName,piRequest.getProductName())
                .eq(StringUtils.isNotBlank(piRequest.getDeviceModel()),DmpProductInfo::getDeviceModel,piRequest.getDeviceModel())
                .eq(null != piRequest.getId(),DmpProductInfo::getId,piRequest.getId())
                .eq(DmpProductInfo::getDeleteFlag,0);
        List<DmpProductInfo> records = page.getRecords();
        List<Integer> productIds = records.stream().map(DmpProductInfo::getId).collect(Collectors.toList());
        List<Map<String, Object>> maps = dmpDeviceInfoService.deviceCollect(productIds);
        List<Map<String, Object>> collectByProduct = dmpDeviceStatusService.getCollectByProduct(productIds);
        records.forEach(s -> {enhanceByDevice(s,maps);
            enhanceByDeviceStatus(s,collectByProduct);
        });
        page = this.page(page,lambdaQuery);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getCurrent(),page.getSize());
    }

    public void enhanceByDevice(DmpProductInfo dmpProductInfo, List<Map<String,Object>> list){
        list.stream()
                .filter(s -> dmpProductInfo.getId().toString().equals(s.get("productId")))
                .findAny()
                .ifPresent(s -> dmpProductInfo.setDeviceCount(Integer.getInteger(s.get("count").toString())));
    }

    public void enhanceByDeviceStatus(DmpProductInfo dmpProductInfo, List<Map<String,Object>> list){
        list.stream()
                .filter(s -> dmpProductInfo.getId().toString().equals(s.get("productId")))
                .findAny()
                .ifPresent(s -> {
                    dmpProductInfo.setDeviceStatusCount(Integer.getInteger(s.get("deviceCount").toString()));
                            dmpProductInfo.setServiceStatusCount(Integer.getInteger(s.get("serviceCount").toString()));
                }
                );
    }

    @Override
    public boolean remove(Integer id) {
        int deviceCount = dmpDeviceInfoService.deviceCountByProductId(id);
        if (deviceCount > 0){
            throw new BusinessException("产品下存在设备,请先删除设备，再删除产品");
        }
        DmpProductInfo dmpProductInfo = this.getById(id);
        Optional.ofNullable(dmpProductInfo).orElseThrow(() -> new BusinessException("产品不存在"));
        dmpProductInfo.setDeleteFlag(1);
        return this.updateById(dmpProductInfo);
    }

    @Override
    public List<Map<String, Object>> collect() {
        return baseMapper.selectCollect();
    }
}
